package com.ggj_linlithgow.gdx.core.battle_state;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL30;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.utils.viewport.StretchViewport;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.deploy_state.UnitCard;
import com.ggj_linlithgow.gdx.core.model.Battle;
import com.ggj_linlithgow.gdx.core.model.Round;
import com.ggj_linlithgow.gdx.core.model.Unit;
import com.ggj_linlithgow.gdx.core.model.UnitType;

public class BattleState extends Stage implements Screen {

	private Display display;
	private Texture background;
	private BitmapFont bitmapFont;
	private String enemy = "Enemy";
	private String player = "Player";
	
	private Map<UnitCard, UnitCard> map = new HashMap<UnitCard, UnitCard>();
	
	private List<UnitCard> playerVsUnits = new ArrayList<UnitCard>();
	private List<UnitCard> enemyVsUnits = new ArrayList<UnitCard>();
	
	private UnitCard selectedPlayerUnit;
	private UnitCard selectedEnemyUnit;
	
	private List<UnitCard> playerUnits = new ArrayList<UnitCard>();
	private List<UnitCard> enemyUnits = new ArrayList<UnitCard>();
	
	private final static Texture archer = new Texture(Gdx.files.internal("image/card_archer.png"));
	private final static Texture cavalry = new Texture(Gdx.files.internal("image/card_cavalry.png"));
	private final static Texture infantry = new Texture(Gdx.files.internal("image/card_infantry.png"));
	int currentRound = 0;
	public BattleState(Display display) {
		this.display = display;
		setViewport(new StretchViewport(Display.WIDTH, Display.HEIGHT));
		background = new Texture(Gdx.files.internal("image/parchment_scroll_background.png"));
		
		bitmapFont = new BitmapFont(Gdx.files.internal("fonts/adventure.fnt"), Gdx.files.internal("fonts/adventure.png"), false);
	}

	@Override
	public void render(float delta) {
		
		Gdx.gl.glClearColor(.0f, .0f, .1f, 1);
		Gdx.gl.glClear(GL30.GL_COLOR_BUFFER_BIT);
		
		getBatch().setProjectionMatrix(getCamera().combined);
		renderer().setProjectionMatrix(getCamera().combined);
		
        getBatch().begin();
        
        getBatch().draw(background, 0, 0, Display.WIDTH, Display.HEIGHT);
        bitmapFont.setColor(Color.RED);
        bitmapFont.draw(getBatch(), enemy, Display.WIDTH/2-(bitmapFont.getBounds(enemy).width/2), Display.HEIGHT-50);
       
        bitmapFont.setColor(Color.GREEN);
        bitmapFont.draw(getBatch(), player, Display.WIDTH/2-(bitmapFont.getBounds(player).width/2), (bitmapFont.getBounds(player).height)+50);
        
        getBatch().end();
        
        Gdx.gl.glEnable(GL30.GL_BLEND);
        renderer().begin(ShapeType.Filled);
        
        renderer().setColor(Color.WHITE);
        renderer().rect(175, Display.HEIGHT/2, Display.WIDTH-325, 5);
        
        for (UnitCard unitCard : playerUnits) {
        	if (unitCard.isTouched() && !isPlayerUnitSelected() && !unitCard.isVs()) {
        		unitCard.setSelected(true);
        		selectedPlayerUnit = unitCard;
        	} 
        	
        	if (unitCard.isSelected()) {
        		renderer().setColor(Color.YELLOW);
        		renderer().rect(unitCard.getX()-15, unitCard.getY()-15, unitCard.getWidth()+25, unitCard.getHeight()+25);
        	}
        	
        	if (Gdx.input.justTouched() && unitCard.isSelected() && !isPointerWithinCards()) {
        		unitCard.setSelected(false);
        	}
        }
        
        for (UnitCard unitCard : enemyUnits) {
        	if (unitCard.isTouched() && !isEnemyUnitSelected() && !unitCard.isVs()) {
        		unitCard.setSelected(true);
        		selectedEnemyUnit = unitCard;
        	} 
        	
        	if (unitCard.isSelected()) {
        		renderer().setColor(Color.RED);
        		renderer().rect(unitCard.getX()-15, unitCard.getY()-15, unitCard.getWidth()+25, unitCard.getHeight()+25);
        	}
        	
        	if (Gdx.input.justTouched() && unitCard.isSelected() && !isPointerWithinCards()) {
        		unitCard.setSelected(false);
        	}
        }
        
        
        
        renderer().end();
        Gdx.gl.glDisable(GL30.GL_BLEND);
        
        Gdx.gl.glEnable(GL30.GL_BLEND);
        renderer().begin(ShapeType.Line);
        
        // only gets called when you do the click
        if (selectedEnemyUnit != null && selectedPlayerUnit != null) {
        	playerVsUnits.add(selectedPlayerUnit);
        	enemyVsUnits.add(selectedEnemyUnit);
        	selectedPlayerUnit.setSelected(false);
        	selectedEnemyUnit.setSelected(false);
        	selectedPlayerUnit.setVs(true);
        	selectedEnemyUnit.setVs(true);
        	selectedEnemyUnit = null;
        	selectedPlayerUnit = null;
        	 for (UnitCard unitCard : enemyUnits) {
        		 unitCard.setSelected(false);
        	 }
        	 for (UnitCard unitCard : playerUnits) {
        		 unitCard.setSelected(false);
        	 }
        }
        
        if (!playerVsUnits.isEmpty() && !enemyVsUnits.isEmpty() && playerVsUnits.get(0) != null && enemyVsUnits.get(0) != null) {
        	renderer().setColor(Color.BLACK);
        	renderer().line(enemyVsUnits.get(0).getOriginX(), enemyVsUnits.get(0).getOriginY(), playerVsUnits.get(0) .getOriginX(), playerVsUnits.get(0) .getOriginY());
        } 
        if (playerVsUnits.size() >= 2 && enemyVsUnits.size() >= 2 && playerVsUnits.get(1) != null && enemyVsUnits.get(1) != null) {
        	renderer().setColor(Color.BLACK);
        	renderer().line(enemyVsUnits.get(1).getOriginX(), enemyVsUnits.get(1).getOriginY(), playerVsUnits.get(1) .getOriginX(), playerVsUnits.get(1) .getOriginY());
        }
        if (playerVsUnits.size() >= 3 && enemyVsUnits.size() >= 3 && playerVsUnits.get(2) != null && enemyVsUnits.get(2) != null) {
        	renderer().setColor(Color.BLACK);
        	renderer().line(enemyVsUnits.get(2).getOriginX(), enemyVsUnits.get(2).getOriginY(), playerVsUnits.get(2) .getOriginX(), playerVsUnits.get(2) .getOriginY());
        }
        if (playerVsUnits.size() >= 4 && enemyVsUnits.size() >= 4 && playerVsUnits.get(3) != null && enemyVsUnits.get(3) != null) {
        	renderer().setColor(Color.BLACK);
        	renderer().line(enemyVsUnits.get(3).getOriginX(), enemyVsUnits.get(3).getOriginY(), playerVsUnits.get(3) .getOriginX(), playerVsUnits.get(3) .getOriginY());
        }
        
        renderer().end();
        Gdx.gl.glDisable(GL30.GL_BLEND);
        
        act();
        draw();
        
        if (playerVsUnits.size() == (int)Math.min(playerUnits.size(), enemyUnits.size())) {
        	
        	final Battle currentBattle = display.getCampaign().getCurrentLocation().getCurrentBattle();
   
        	boolean outcome = false;

        		System.out.println("===== DING DING, ROUND "+currentRound+" ======");
        		currentBattle.addRound(new Round(getUnitsFromPlayer(), getUnitsFromEnemy(), playerUnits, enemyUnits));

        		outcome = currentBattle.fight(currentRound);
        		if (!outcome) {
        			rebase();
        		}
        		currentRound++;
        	
           if (currentBattle.isBattleComplete()) {
        	   currentRound = 0;
        	if (outcome) {
        		System.out.println("WINNING!");
        		display.getCampaign().getCurrentLocation().setLocationToWon();
        		display.gotoMap();
        	} else {
        		System.out.println("LOSING!");
        		display.gotoMap();
        	}
           }
        }
	}
	
	private List<Unit> getUnitsFromPlayer() {
		List<Unit> units = new ArrayList<Unit>();
		for (UnitCard card : playerVsUnits) {
			units.add(card.getUnit());
		}
		return units;
	}
	
	private List<Unit> getUnitsFromEnemy() {
		List<Unit> units = new ArrayList<Unit>();
		for (UnitCard card : enemyVsUnits) {
			units.add(card.getUnit());
		}
		return units;
	}
	
	public boolean isPointerWithinCards() {
		for (UnitCard unitCard : playerUnits) {
			 if (unitCard.isPointerWithinCard()) {
				 return true;
			 }
		 }
		 for (UnitCard unitCard : enemyUnits) {
			 if (unitCard.isPointerWithinCard()) {
				 return true;
			 }
		 }
		 return false;
	}

	public boolean isPlayerUnitSelected() {
		 for (UnitCard unitCard : playerUnits) {
			 if (unitCard.isSelected() ) {
				 return true;
			 }
		 }
		 return false;
	}
	
	public boolean isEnemyUnitSelected() {
		 for (UnitCard unitCard : enemyUnits) {
			 if (unitCard.isSelected() ) {
				 return true;
			 }
		 }
		 return false;
	}
	
	@Override
	public void resize(int width, int height) {
		getViewport().update(width, height, true);
		getCamera().position.set(Display.WIDTH/2, Display.HEIGHT/2, 0);
	}

	private void rebase() {
		float alpha = 200;
		float origin = 350;
		int count = 0;
		
		for (UnitCard card : playerUnits) {
			card.reset();
		}
		
		for (UnitCard card : enemyUnits) {
			card.reset();
		}
		
		clear(); // clear the stage
		playerVsUnits.clear();
		enemyVsUnits.clear();
		playerUnits.clear();
		enemyUnits.clear();
		
		for (final Unit playerUnit : display.getCampaign().getCurrentLocation().getCurrentBattle().getAvailableGoodGuys()) {
			System.out.println("REBASE: Available good unit: "+playerUnit.getType()+" ("+playerUnit.getTroopCount()+")");
			if (UnitType.ARCHER.equals(playerUnit.getType())) {
				UnitCard unitCard = new UnitCard(origin+(count*alpha), 150, 150, 200, archer,bitmapFont , playerUnit, count);
				playerUnits.add(unitCard);
				addActor(unitCard);
			} else if (UnitType.CAVALRY.equals(playerUnit.getType())) {
				UnitCard unitCard = new UnitCard(origin+(count*alpha), 150, 150, 200, cavalry,bitmapFont , playerUnit, count);
				playerUnits.add(unitCard);
				addActor(unitCard);
			} else if (UnitType.INFANTRY.equals(playerUnit.getType())) {
				UnitCard unitCard = new UnitCard(origin+(count*alpha), 150, 150, 200, infantry,bitmapFont , playerUnit, count);
				playerUnits.add(unitCard);
				addActor(unitCard);
			}
			count++;
		}
		
		alpha = 200;
		origin = 350;
		count = 0;
		
		for (Unit playerUnit :  display.getCampaign().getCurrentLocation().getCurrentBattle().getAvailableBadGuys()) {
			System.out.println("REBASE: Available bad unit: "+playerUnit.getType()+" ("+playerUnit.getTroopCount()+")");
			if (UnitType.ARCHER.equals(playerUnit.getType())) {
				UnitCard unitCard = new UnitCard(origin+(count*alpha), Display.HEIGHT-350, 150, 200, archer,bitmapFont , playerUnit, count);
				enemyUnits.add(unitCard);
				addActor(unitCard);
			} else if (UnitType.CAVALRY.equals(playerUnit.getType())) {
				UnitCard unitCard = new UnitCard(origin+(count*alpha), Display.HEIGHT-350, 150, 200, cavalry,bitmapFont , playerUnit, count);
				enemyUnits.add(unitCard);
				addActor(unitCard);
			} else if (UnitType.INFANTRY.equals(playerUnit.getType())) {
				UnitCard unitCard = new UnitCard(origin+(count*alpha), Display.HEIGHT-350, 150, 200, infantry,bitmapFont , playerUnit, count);
				enemyUnits.add(unitCard);
				addActor(unitCard);
			}
			count++;
		}
	}
	
	@Override
	public void show() {

		
		rebase();
	}

	@Override
	public void hide() {
		
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		super.dispose();
	}
	
	private ShapeRenderer renderer() {
		return display.getRenderer();
	}
}
