package com.ggj_linlithgow.gdx.core.deploy_state;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.ggj_linlithgow.gdx.core.model.Unit;

public class CardHandler {

	private int unitsAvailable = 0;
	private LoadBattleActors loadBattleActors;
	private int choosenUnits;
	private BitmapFont bitmapFont;
	
	private List<CardPlaceHolder> cards = new ArrayList<CardPlaceHolder>();
	
	public CardHandler(int unitsAvailable, Texture emptyCard, DeployState display) {
		this.unitsAvailable = unitsAvailable;
		bitmapFont = new BitmapFont(Gdx.files.internal("fonts/adventure.fnt"), Gdx.files.internal("fonts/adventure.png"), false);
		loadBattleActors = new LoadBattleActors(display, bitmapFont);
		
		float alpha = 200;
		float origin = 350;
		
		cards.add(new CardPlaceHolder(origin, 150));
		cards.add(new CardPlaceHolder(origin+alpha, 150));
		cards.add(new CardPlaceHolder(origin+(alpha*2), 150));
		cards.add(new CardPlaceHolder(origin+(alpha*3), 150));
	}
	
	public void draw(Batch batch) {
		
		for (CardPlaceHolder placeHolder : cards) {
			placeHolder.draw(batch);
		}
		
//		String text = "Choosen Unit: " + choosenUnits;
//		bitmapFont.draw(batch, text, (Display.WIDTH/2)-bitmapFont.getBounds(text).width/2, (Display.HEIGHT)-bitmapFont.getBounds(text).height/2);
	}
	
	public void update() {
		
		if (unitsAvailable == 4) {
			for (UnitCard card : loadBattleActors.getUnitCards()) {
				if (card.isCardOverLine() && !card.isLocked()) {
					
					for (CardPlaceHolder placeHolder : cards) {
						if (placeHolder.isSlotAvailable() && cardNotAlreadyPositioned(card)) {
							placeHolder.updateCardToPlaceHolderPosition(card);
							choosenUnits++;
						}
					}
					
					if (card.getLockedXPos() != 0 && card.getLockedYPos() != 0) {
						card.setX(card.getLockedXPos());
						card.setY(card.getLockedYPos());
					}
					
				}
			}
		}
		
		if (isCardSelected()) {
			for (UnitCard card : loadBattleActors.getUnitCards()) {
				if (!card.isLocked()) {
					card.setDisableMovement(true);
				}
			}
		} else {
			for (UnitCard card : loadBattleActors.getUnitCards()) {
				if (card.isDisableMovement()) {
					card.setDisableMovement(false);
				} 
			}
		}
		
		for (CardPlaceHolder placeHolder : cards) {
			if (placeHolder.getSlotCard() != null && !placeHolder.getSlotCard().isCardOverLine()) {
				placeHolder.removeCardFromSlot();
				choosenUnits--;
			}
		}
		
	}
	
	private boolean isCardSelected() {
		for (UnitCard card : loadBattleActors.getUnitCards()) {
			if (card.isLocked()) {
				return true;
			}
		}
		return false;
	}
	
	private boolean cardNotAlreadyPositioned(UnitCard card) {
		for (CardPlaceHolder placeHolder : cards) {
			if (card.equals(placeHolder.getSlotCard())) {
				return false;
			}
		}
		return true;
	}
	
	public List<UnitCard> getCards() {
		return loadBattleActors.getUnitCards();
	}
	
	public BitmapFont getBitmapFont() {
		return bitmapFont;
	}
	
	public void begin() {
		loadBattleActors.begin();
	}
	
	public void reset() {
		for (CardPlaceHolder placeHolder : cards) {
			placeHolder.removeCardFromSlot();
			choosenUnits = 0;
		}
		loadBattleActors.reset();
	}
	
	public void dispose() {
		loadBattleActors.dispose();
	}
	
	public List<Unit> getSelectedUnits() {
		List<Unit> selectedUnits = new ArrayList<Unit>();
		for (CardPlaceHolder placeHolder : cards) {
			selectedUnits.add(placeHolder.getSlotCard().getUnit());
		}
		return selectedUnits;
	}
	
	public boolean isAllUnitsSelected() {
		for (CardPlaceHolder placeHolder : cards) {
			if (placeHolder.isSlotAvailable()) {
				return false;
			}
		}
		return true;
	}
}
