package com.ggj_linlithgow.gdx.core.deploy_state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL30;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.utils.viewport.StretchViewport;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.model.Battle;
import com.ggj_linlithgow.gdx.core.model.Location;
import com.ggj_linlithgow.gdx.core.model.Round;
import com.ggj_linlithgow.gdx.core.model.Unit;

public class DeployState extends Stage implements Screen {

	private final Display display;
	private Texture background, arrow;
	private float xPos, yPos, width, height;
	private CardHandler cardHandler;
	private Vector3 unproject = new Vector3(0, 0, 0);
	
	public DeployState(Display display) {
		this.display = display;
		setViewport(new StretchViewport(Display.WIDTH, Display.HEIGHT));
		background = new Texture(Gdx.files.internal("image/parchment_scroll_background.png"));
		
		arrow = new Texture(Gdx.files.internal("image/right-arrow-hi.png"));
		this.xPos = Display.WIDTH-300;
		this.yPos = 100;
		this.width = 150;
		this.height = 150;
	
		cardHandler = new CardHandler(4, null, this);
	}
	
	@Override 
	public void render(float delta) {
	
		Gdx.gl.glClearColor(.0f, .0f, .1f, 1);
		Gdx.gl.glClear(GL30.GL_COLOR_BUFFER_BIT);
		
		getBatch().setProjectionMatrix(getCamera().combined);
		renderer().setProjectionMatrix(getCamera().combined);
		
        getBatch().begin();
        
        getBatch().draw(background, 0, 0, Display.WIDTH, Display.HEIGHT);
        cardHandler.draw(getBatch());
        getBatch().draw(arrow, xPos, yPos, width, height);
        
        Location currentLocation = display.getCampaign().getCurrentLocation();
		Unit scountedOne = currentLocation.getScoutedGuys().get(0);
        Unit scountedTwo = currentLocation.getScoutedGuys().get(1);
		String text = scountedOne.getType().getName() + " (" + scountedOne.getTroopCount() + ") +" + scountedTwo.getType().getName() + " (" + scountedTwo.getTroopCount() + ")";
        cardHandler.getBitmapFont().draw(getBatch(), text, (Display.WIDTH/2)-cardHandler.getBitmapFont().getBounds(text).width/2, (Display.HEIGHT)-cardHandler.getBitmapFont().getBounds(text).height/2);
       
        getBatch().end();
        
        Gdx.gl.glEnable(GL30.GL_BLEND);
        renderer().begin(ShapeType.Filled);
        
        renderer().setColor(Color.WHITE);
        renderer().rect(175, Display.HEIGHT/2, Display.WIDTH-325, 5);
        
        renderer().end();
        Gdx.gl.glDisable(GL30.GL_BLEND);
        
        cardHandler.update();
        
        act();
		draw();
		
		unproject.set(Gdx.input.getX(), Gdx.input.getY(), 0);
		getCamera().unproject(unproject);
		if (unproject.x > xPos && unproject.x < xPos+width 
				&& unproject.y > yPos && unproject.y < yPos+height 
				&& Gdx.input.justTouched() && cardHandler.isAllUnitsSelected()) {
			
			final Battle battle = new Battle();
			battle.setInitalBattleState(cardHandler.getSelectedUnits(), currentLocation.getBadGuys());
			
			currentLocation.addBattle(battle);
			display.gotoBattle();
		}
        
	}

	@Override
	public void resize(int width, int height) {
		getViewport().update(width, height, true);
		getCamera().position.set(Display.WIDTH/2, Display.HEIGHT/2, 0);
	}

	@Override
	public void show() {
		cardHandler.begin();
	}

	@Override
	public void hide() {
		cardHandler.reset();
	}

	@Override
	public void pause() {
	}

	@Override
	public void resume() {
	}

	@Override
	public void dispose() {
		cardHandler.dispose();
		background.dispose();
		super.dispose();
	}
	
	protected Display getDisplay() {
		return display;
	}
	
	private ShapeRenderer renderer() {
		return display.getRenderer();
	}
}
