package com.ggj_linlithgow.gdx.core.deploy_state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.model.Unit;

public class UnitCard extends Actor {
	
	private Texture ticket;
	private Vector3 unproject = new Vector3(0, 0, 0);
	private boolean locked = false, selected = false, vs;
	private boolean disableMovement = false;
	private float lockedXPos, lockedYPos;
	private LoadBattleActors battleActors;
	private float originWidth, originHeight;
	private BitmapFont bitmapFont;
	
	private Unit unit;
	private int id;
	
	public UnitCard(float xPos, float yPos, float width, float height, Texture ticket, BitmapFont bitmapFont, Unit unit, int id) {
		setX(xPos);
		setY(yPos);
		setOriginX(xPos+(width/2));
		setOriginY(yPos+(height/2));
		this.originWidth = width;
		this.originHeight = height;
		setWidth(width);
		setHeight(height);
		this.ticket = ticket;
		this.bitmapFont = bitmapFont;
		this.unit = unit;
		this.id = id;
	}
	
	public UnitCard(float xPos, float yPos, float width, float height, Texture ticket, LoadBattleActors loadBattleActors, BitmapFont bitmapFont, Unit unit) {
		setX(xPos);
		setY(yPos);
		this.originWidth = width;
		this.originHeight = height;
		setWidth(width/2);
		setHeight(height/2);
		this.ticket = ticket;
		this.battleActors = loadBattleActors;
		this.bitmapFont = bitmapFont;
		this.unit = unit;
	}
	
	@Override
	public void act(float delta) {
		unproject.set(Gdx.input.getX(), Gdx.input.getY(), 0);
		getStage().getCamera().unproject(unproject);
		
		if (!isDisableMovement() && !locked && (unproject.x > getX() && unproject.x < getX()+getWidth() 
				&& unproject.y > getY() && unproject.y < getY()+getHeight() 
				&& Gdx.input.justTouched()) && !isCardAlreadySelect()) {
			
			locked = true;
		}
		
		if (locked && !Gdx.input.isTouched()) {
			
			locked = false;
		} 
		
		if (locked) {
			setX(unproject.x-(getWidth()/2));
			setY(unproject.y-(getHeight()/2));
		}
	}
	
	public boolean isTouched() {
		if ((unproject.x > getX() && unproject.x < getX()+getWidth() 
				&& unproject.y > getY() && unproject.y < getY()+getHeight() 
				&& Gdx.input.isTouched())) {
			
			return true;
		}
		
		return false;
	}
	
	public boolean isPointerWithinCard() {
		if ((unproject.x > getX() && unproject.x < getX()+getWidth() 
				&& unproject.y > getY() && unproject.y < getY()+getHeight())) {
			
			return true;
		}
		
		return false;
	}
	
	public void setDisableMovement(boolean disableMovement) {
		this.disableMovement = disableMovement;
	}
	
	public boolean isDisableMovement() {
		return disableMovement;
	}
	
	public boolean isLocked() {
		return locked;
	}
	
	public boolean isSelected() {
		return selected;
	}
	
	public void setSelected(boolean selected) {
		this.selected = selected;
	}
	
	public boolean isCardOverLine() {
		return getY() > (Display.HEIGHT/2)-getHeight();
	}
	
	@Override
	public void setX(float x) {
		super.setX(x);
	}
	
	@Override
	public void setY(float y) {
		super.setY(y);
	}
	
	@Override
	public float getHeight() {
		return super.getHeight();
	}
	
	@Override
	public float getWidth() {
		return super.getWidth();
	}
	
	public void setLocked(boolean locked) {
		this.locked = locked;
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		
		batch.draw(ticket, getX(), getY(), getWidth(), getHeight());
		if (bitmapFont != null) {
			String unit = unitCount() + "";
			bitmapFont.setColor(Color.YELLOW);
			bitmapFont.draw(batch, unit, (getX()+(getWidth()/2))-(bitmapFont.getBounds(unit).width/2), (getY()+(getHeight()/2)));
			bitmapFont.setColor(Color.WHITE);
		}
	}

	public float getLockedXPos() {
		return lockedXPos;
	}

	public void setLockedXPos(float lockedXPos) {
		this.lockedXPos = lockedXPos;
	}

	public float getLockedYPos() {
		return lockedYPos;
	}

	public void setLockedYPos(float lockedYPos) {
		this.lockedYPos = lockedYPos;
	}
	
	private boolean isCardAlreadySelect() {
		if (battleActors == null || battleActors.getUnitCards() == null) return true;
		for (UnitCard card : battleActors.getUnitCards()) {
			if (!card.equals(this) && card.isLocked()) {
				return true;
			}
		}
		return false;
	}

	public float getOriginWidth() {
		return originWidth;
	}
	
	public float getOriginHeight() {
		return originHeight;
	}
	
	public int unitCount() {
		return unit.getTroopCount();
	}
	
	public Unit getUnit() {
		return unit;
	}
	
	public int getId() {
		return id;
	}
	
	public void setVs(boolean vs) {
		this.vs = vs;
	}
	
	public boolean isVs() {
		return vs;
	}

	public void reset() {
		locked = false;
		selected = false;
		vs = false;
	}
}
