package com.ggj_linlithgow.gdx.core.map_state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.GL30;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.badlogic.gdx.utils.viewport.StretchViewport;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.model.Location;
import com.ggj_linlithgow.gdx.core.terrain.Background;

public class MapState implements Screen {

	private final Display display;

	private Stage stage;
	private Stage backStage = new Stage();
    private final Texture background; 

    private BitmapFont titleText;
	private String titleTextChars = "What do we do now?";
    private Actor zot;
    private final int zotsize = 256;
    
	public MapState(final Display display) {
		stage = new Stage();
		this.display = display;

		stage.setViewport(new StretchViewport(1600,900));
		backStage.setViewport(new StretchViewport(1600,900));
		titleText = new BitmapFont(Gdx.files.internal("fonts/adventure.fnt"), Gdx.files.internal("fonts/adventure.png"), false);
		background = new Texture(Gdx.files.internal("image/54c46c0990181308.png"));

		// first background
		zot = new Background("image/zoneofterror.png", display, 512, 512);
		zot.setSize(zotsize, zotsize);
		zot.setOrigin(zotsize / 2, zotsize / 2);
		zot.setTouchable(Touchable.enabled);
		backStage.addActor(zot);
		
		for (final Location l : display.campaign.getLocations()) {
			addVillage(l);
		}
	}
	
	void addVillage(final Location location) {
		final Village v = new Village(location, display);
		stage.addActor(v);
		stage.addActor(v.getTextName());
		v.getScoutReport().setVisible(false);
		stage.addActor(v.getScoutReport());
	}

	@Override
	public void render(float delta) {
	        Gdx.gl.glClearColor(0.5f, 0.5f, 0, 1); // don't clear the colour bit if full screen overdraw of course :)
	        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
	
	        backStage.getBatch().setProjectionMatrix(backStage.getCamera().combined);
			renderer().setProjectionMatrix(backStage.getCamera().combined);
			
	        backStage.getBatch().begin();
	        backStage.getBatch().draw(background, 0, 0, 1600, 900);
	        
	        titleText.setColor(Color.YELLOW);
	        titleText.draw(backStage.getBatch(), titleTextChars, (Gdx.graphics.getWidth()/2)-titleText.getBounds(titleTextChars).width/2, (Gdx.graphics.getHeight())-titleText.getBounds(titleTextChars).height/2);
	        
	        backStage.getBatch().end();

			if (display.campaign.getCurrentLocation() != null) {
				zot.setVisible(true);
				zot.setPosition(display.campaign.getCurrentLocation().getX()-(zotsize>>1), display.campaign.getCurrentLocation().getY()-(zotsize>>1));
			} else {
				zot.setVisible(false);
			}
	        zot.rotateBy(10*delta);
	        backStage.act(delta);
	        backStage.draw();

	        //drawLinesBetweenVillages(); // or not :)
	        
	        stage.act(delta);
			stage.draw();
			
			
	}
	
	/**
	 * binned
	 */
	private void drawLinesBetweenVillages() {
		Gdx.gl.glEnable(GL30.GL_BLEND);
		Gdx.gl20.glLineWidth(5); // don't use this. it's bad.
        
        renderer().begin(ShapeType.Line);

        // draw lines between villages
        if (!Village.getAllVillages().isEmpty()) {
        	Village root = Village.getAllVillages().get(0);
        	for (final Village v : Village.getAllVillages()) {
        		if (v == root) continue;
        		if (v == Village.getAllVillages().get(Village.getAllVillages().size()-1)) {
        			renderer().setColor(Color.YELLOW);
        		} else if (Village.getAllVillages().indexOf(v) < display.campaign.getLocations().indexOf(display.campaign.getCurrentLocation())) {
        			renderer().setColor(Color.GREEN);
        		} else {
        			renderer().setColor(Color.RED);
        		}
        		
        		renderer().line(root.x, root.y, v.x, v.y);
        		root = v;
        	}
        }
        
        renderer().end();
        Gdx.gl.glDisable(GL30.GL_BLEND);
	}
	
	private ShapeRenderer renderer() {
		return display.getRenderer();
	}

	@Override
	public void resize(int width, int height) {
		backStage.getViewport().update(width, height, true);
		backStage.getCamera().position.set(1600/2, 900/2, 0);
		stage.getViewport().update(width, height, true);
		stage.getCamera().position.set(1600/2, 900/2, 0);
	}

	@Override
	public void show() {
		//SoundManager.playLoadingTune();
		Gdx.input.setInputProcessor(stage);
	}

	@Override
	public void hide() {
		Gdx.input.setInputProcessor(null);
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		if (stage != null) stage.dispose();
		if (backStage != null) backStage.dispose();
	}
}
