package com.ggj_linlithgow.gdx.core.map_state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.model.Location;

public class ScoutReport extends Actor {
	private static Texture texture = new Texture(Gdx.files.internal("image/report.png"));
	final Display display;
	int x;
	int y;
	private Location location;
	
	private BitmapFont titleText;
	private String titleTextChars = "Scout Report";

	// unless you were barmy you'd write this properly. 
	private BitmapFont labelLocation;
	private String labelLocationName = "Location";
	
	private BitmapFont labelEnemy;
	private String labelEnemyName = "Enemy";
	
	private BitmapFont labelNotes;
	private String labelNotesName = "Notes";	
	
	private BitmapFont labelLocationText;
	private String labelLocationValue = "";
	
	private BitmapFont labelEnemyTextA;
	private String labelEnemyValueA = "";
	
	private BitmapFont labelEnemyTextB;
	private String labelEnemyValueB = "";
	
	private BitmapFont labelNotesText;
	private String labelNotesValue = "";
	
	public ScoutReport (final Display display) {
		this.display = display;
		titleText = new BitmapFont(Gdx.files.internal("fonts/adventure.fnt"), Gdx.files.internal("fonts/adventure.png"), false);
		titleText.setColor(Color.BLACK);
		
		labelLocation = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelLocation.setColor(Color.BLACK);
		
		labelEnemy = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelEnemy.setColor(Color.BLACK);
		
		labelNotes = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelNotes.setColor(Color.BLACK);
		
		labelLocationText = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelLocationText.setColor(Color.YELLOW);
		
		labelEnemyTextA = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelEnemyTextA.setColor(Color.RED);
		labelEnemyTextB = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelEnemyTextB.setColor(Color.RED);
		
		labelNotesText = new BitmapFont(Gdx.files.internal("fonts/adventure-28.fnt"), Gdx.files.internal("fonts/adventure-28.png"), false);
		labelNotesText.setColor(Color.BLUE);
		
		// set width so that it can actually be TOUCHED

		setWidth(texture.getWidth()*.75f);
		setHeight(texture.getHeight()*.75f);
		
		setTouchable(Touchable.enabled);

		this.addListener(new InputListener() {
		public boolean touchDown(InputEvent event, float x, float y,
				int pointer, int button) {
			System.out.println("touch on scoutreport");
			//selected.play();
			display.campaign.setCurrentLocation(location);
			display.gotoDeploy();
			hide();
			return true;
		}});
	}
	
	public void hide() {
		setVisible(false);

	}
	
	public void show() {
		setVisible(true);

	}
	
	public void setReportDetails(final Location location) {
		this.location = location;
		display.campaign.setCurrentLocation(location);
		this.x = (Gdx.graphics.getWidth()>>1)-(int)(getWidth()/2);
		this.y = (Gdx.graphics.getHeight()>>1)-(int)(getHeight()/2);
		
		labelLocationValue = location.getName();

		if (location.getScoutedGuys().size() > 0) {
			labelEnemyValueA = location.getScoutedGuys().get(0).getTroopCount()+"x "+location.getScoutedGuys().get(0).getType().name();
		}
		
		if (location.getScoutedGuys().size() > 1) {
			labelEnemyValueB = location.getScoutedGuys().get(1).getTroopCount()+"x "+location.getScoutedGuys().get(1).getType().name();
		}
		
	    labelNotesValue = location.badGuys.size() > 2 ? "+"+(location.badGuys.size()-2)+" more unit(s)" : "this is all of them";
	
		setBounds(x, y, getWidth(), getHeight());
	}
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		super.draw(batch, parentAlpha);
		batch.draw(texture, x, y, getWidth(), getHeight());
		titleText.draw(batch, titleTextChars,
				(Gdx.graphics.getWidth()/2)-titleText.getBounds(titleTextChars).width/2,
				(getHeight())-(titleText.getBounds(titleTextChars).height*-1));
		
		labelLocation.draw(batch, labelLocationName,
				(Gdx.graphics.getWidth()/2)-labelLocation.getBounds(labelLocationName).width,
				(getHeight())-(labelLocation.getBounds(labelLocationName).height*2));
		
		labelEnemy.draw(batch, labelEnemyName,
				(Gdx.graphics.getWidth()/2)-labelEnemy.getBounds(labelEnemyName).width,
				(getHeight())-(labelEnemy.getBounds(labelEnemyName).height*4));
		
		labelNotes.draw(batch, labelNotesName,
				(Gdx.graphics.getWidth()/2)-labelNotes.getBounds(labelNotesName).width,
				(getHeight())-(labelNotes.getBounds(labelNotesName).height*8));
		
		labelLocationText.draw(batch, labelLocationValue,
				(Gdx.graphics.getWidth()/2)-labelLocationText.getBounds(labelLocationValue).width/2,
				(getHeight())-(labelLocationText.getBounds(labelLocationValue).height*3));
		
		labelEnemyTextA.draw(batch, labelEnemyValueA,
				(Gdx.graphics.getWidth()/2)-labelEnemyTextA.getBounds(labelEnemyValueA).width/2,
				(getHeight())-(labelEnemyTextA.getBounds(labelEnemyValueA).height*5));
		labelEnemyTextB.draw(batch, labelEnemyValueB,
				(Gdx.graphics.getWidth()/2)-labelEnemyTextB.getBounds(labelEnemyValueB).width/2,
				(getHeight())-(labelEnemyTextB.getBounds(labelEnemyValueB).height*6));
		
		labelNotesText.draw(batch, labelNotesValue,
				(Gdx.graphics.getWidth()/2)-labelNotesText.getBounds(labelNotesValue).width/2,
				(getHeight())-(labelNotesText.getBounds(labelNotesValue).height*9));
	}
}
