package com.ggj_linlithgow.gdx.core.map_state;

import java.util.ArrayList;
import java.util.List;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Sound;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.FontManager;
import com.ggj_linlithgow.gdx.core.actor.TextActor;
import com.ggj_linlithgow.gdx.core.model.Location;

public class Village extends Actor {
	private static Texture texture = new Texture(Gdx.files.internal("image/town2.png"));
	private static Texture shadow = new Texture(Gdx.files.internal("image/town1.png"));

	final Display display;
	TextActor textName;
	int unitsInCamp;
	
	// there is only one scroll, we just send it different values from the scout
	private static ScoutReport scoutReport;
	
	final Sound selected = Gdx.audio.newSound(Gdx.files
			.internal("sound/188815__porphyr__battle-horn.ogg"));
	
	private static final List<Village> villages = new ArrayList<>();
	public static List<Village> getAllVillages() {
		return villages;
	}

	final Location location;
	final int x;
	final int y;
	
	public Village (final Location location, final Display display) {
		this.display = display; 
		this.location = location;
		if (scoutReport == null) {
			scoutReport = new ScoutReport(display);
		}
		this.x = location.getX()-(texture.getWidth()>>1);
		this.y = location.getY()-(texture.getHeight()>>1);
		unitsInCamp = 4; // default for now
		
		// set width so that it can actually be TOUCHED
		setWidth(texture.getWidth());
		setHeight(texture.getHeight());
		
		// AND set bounds because touch events are finiky that way - jeez
		setBounds(this.x, this.y, getWidth(), getHeight());
		setTouchable(Touchable.enabled);
		
		villages.add(this);
		// next line will produce java.lang.ArrayIndexOutOfBoundsException: 21 if you place too many villages
		textName = new TextActor(location.getName(), x, y-32, FontManager.getNormalLabel());
		textName.setX(x+
				32-(textName.getWidth()/2));
		textName.setTouchable(Touchable.disabled);
		
		this.addListener(new InputListener() {
		public boolean touchDown(InputEvent event, float x, float y,
				int pointer, int button) {
			System.out.println("VILLAGE INFO/SCOUT REPORT: "+location.toString());
			//selected.play(); // sounds like a whale. don't do this
			
			// toggle visibility
				if (!scoutReport.isVisible()) {
					scoutReport.setReportDetails(location);
					scoutReport.show();
				} else {
					scoutReport.hide();
				}
				
			
			return true;
		}});
	}
		
	TextActor getTextName() {
		return textName;
	}
	
	ScoutReport getScoutReport() {
		return scoutReport;
	}
	
	@Override
	public void act(float delta) {
		super.act(delta);
	}
	
	@Override
	public void draw(Batch batch, float parentAlpha) {
		super.draw(batch, parentAlpha);
		
		batch.draw(shadow, x-4, y+4);
		
		if (location == display.campaign.getCurrentLocation()) {
			if (display.campaign.getCurrentLocation().isLocationWon()) {
				batch.setColor(Color.GREEN);
			} else {
				batch.setColor(Color.YELLOW);
			}
		} else {
			batch.setColor(Color.RED);
		}
		// TODO: or already visited AND WON = GREEN
		
//		if (this == Village.getAllVillages().get(Village.getAllVillages().size()-1)) {
//			 batch.setColor(Color.YELLOW);
//		} else if (Village.getAllVillages().indexOf(this) < display.campaign.getLocations().indexOf(display.campaign.getCurrentLocation())) {
//			 batch.setColor(Color.GREEN);
//		} else {
//			 batch.setColor(Color.RED);
//		}
		 
          // batch.setColor(color.r, 1.0f, color.b, color.a * parentAlpha);
		batch.draw(texture, x, y);
		batch.setColor(Color.WHITE);
	}
}
