package com.ggj_linlithgow.gdx.core.menu_state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.Touchable;
import com.ggj_linlithgow.gdx.core.Display;
import com.ggj_linlithgow.gdx.core.sound.SoundManager;
import com.ggj_linlithgow.gdx.core.terrain.Background;

public class MenuState implements Screen {

	private final Display display;
	private Stage backStage;
	
	public MenuState(final Display display) {
		this.display = display;
		this.backStage = new Stage();
		
		// first background
		Actor background = new Background("image/title.jpg", display, 785, 441);
		int w = Gdx.graphics.getWidth();
		int h = Gdx.graphics.getHeight();
		background.setSize(w, h);
		background.setOrigin(w / 2, h / 2);
		background.setPosition(0, 0);
		background.setTouchable(Touchable.enabled);
		backStage.addActor(background);
		
		backStage.addListener(new InputListener() {
			public boolean touchDown(InputEvent event, float x, float y,
					int pointer, int button) {
				display.gotoMap();
				return true; // false = propagate (consume = false)
			}});
		
	}

	@Override
	public void render(float delta) {
	     Gdx.gl.glClearColor(0.5f, 0.5f, 0, 1); // don't clear the colour bit if full screen overdraw of course :)
	        Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);
	
	        backStage.act(delta);
	        backStage.draw();
	}

	@Override
	public void resize(int width, int height) {
		
	}

	@Override
	public void show() {
		SoundManager.playLoadingTune();
		Gdx.input.setInputProcessor(backStage);
	}

	@Override
	public void hide() {
		
	}

	@Override
	public void pause() {
		
	}

	@Override
	public void resume() {
		
	}

	@Override
	public void dispose() {
		
	}
}
