package com.ggj_linlithgow.gdx.core.model;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Location {

	private static final int UNITS_PER_LOCATION = 4;
	private final String name;
	private final String notes;
	private int x,y;
	public final List<Unit> badGuys = new ArrayList<>(UNITS_PER_LOCATION);
	private final List<Battle> battles = new ArrayList<>();
	
	private boolean won = false;

	public Location (final String name, final int x, final int y, final String notes) {
		this.name = name;
		this.x = x;
		this.y = y;
		this.notes = notes;
		generateUnits();
	}
	
	public String getNotes() {
		return notes;
	}
	
	/**
	 * NOT THE LOCATION BAD GUYS, JUST THESE ONES!
	 */
	public List<Unit> getScoutedGuys() {
		return badGuys.subList(0, badGuys.size()>>1);
	}
	
	private void generateUnits() {
		final Random r = new Random();
		for (int i=0; i < UNITS_PER_LOCATION; i++) {
			badGuys.add(new Unit(UnitType.values()[r.nextInt(UnitType.values().length)], "Blackwatch", 10));// r.nextInt(3)+8));
		}
	}
	
	public void setLocationToWon() {
		won = true;
	}
	
	public boolean isLocationWon() {
		return won;
	}
	
	/**
	 * Record battle
	 */
	public void addBattle(final Battle battle) {
		battles.add(battle);
	}
	
	public Battle getCurrentBattle() {
		return battles.get(0);
	}

	public String getName() {
		return name;
	}
	
	public int getX() {
		return x;
	}
	
	public int getY() {
		return y;
	}
	
	public List<Unit> getBadGuys() {
		return badGuys;
	}

	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder("Location [name=" + name + ", x=" + x + ", y=" + y+ ", badGuys=");
		for (Unit u : badGuys) {
			sb.append (u.toString());
		}
		sb.append (']');
		return sb.toString();
	}	
}
