package com.ggj_linlithgow.gdx.core.model;

import java.util.ArrayList;
import java.util.List;

import com.ggj_linlithgow.gdx.core.deploy_state.UnitCard;

public class Round {

	private List<Unit> goodGuysPicked = new ArrayList<Unit>(4);
	private List<Unit> badGuysPicked = new ArrayList<Unit>(4);
	private List<SkirmishResult> skirmishResults = new ArrayList<SkirmishResult>();
	private List<UnitCard> goodGuysAll = new ArrayList<>(4);
	private List<UnitCard> badGuysAll = new ArrayList<>(4);
	
	public Round(List<Unit> goodGuysPicked, List<Unit> badGuysPicked, List<UnitCard> goodGuysAll, List<UnitCard> badGuysAll) {
		this.goodGuysPicked = goodGuysPicked;
		this.badGuysPicked = badGuysPicked;
		this.goodGuysAll = goodGuysAll;
		this.badGuysAll = badGuysAll;
		setEnemies();
	}
	
	private void setEnemies() {
		
		for (Unit badGuy : badGuysPicked) {
			badGuy.setEnemy();
		}
	}

	public void fight() {
		
		int skirmNum = (int)Math.max(goodGuysAll.size(), badGuysAll.size());
		System.out.println("Running "+skirmNum+" skirmishes");
		for (int skirmishNumber=0; skirmishNumber < skirmNum; skirmishNumber++) {
			
			Unit goodGuy = null;
			Unit badGuy = null;
					
			Unit winner = null;
			Unit loser = null;
			
		    if(skirmishNumber < goodGuysPicked.size()) {
			    goodGuy = goodGuysPicked.get(skirmishNumber);
		    } else {
		    	winner = badGuy;
		    }
			
		    if(skirmishNumber < badGuysPicked.size()) {
		    	badGuy = badGuysPicked.get(skirmishNumber);
		    } else {
		    	winner = goodGuy;
		    }
						
			if (winner == null) {
				winner = determineWinner(goodGuy, badGuy);
				loser = (winner.equals(goodGuy)) ? badGuy : goodGuy;
			}
			
			System.out.println("Skirmish Details" + (goodGuy != null ? goodGuy.getType().getName() : null)+ " (" + (goodGuy != null ? goodGuy.getTroopCount() : null) + ") vs. " + (badGuy != null ? badGuy.getType().getName() : null) + " (" + (badGuy != null ? badGuy.getTroopCount() : null) + ")");
			
			//Skirmish result will perform unit updates i.e. eliminate units/decrement killed troops, probably not the right place but.....
			SkirmishResult result = new SkirmishResult(winner, loser, loser == null ? winner.getTroopCount() : determineTroopsRemaining(winner, loser));
			result.printReport();
			
			skirmishResults.add(result);
		}
	}
	
	public boolean isEnemyDefeated() {
		
		if (skirmishResults == null || skirmishResults.size() < 1) {
			return false;
		}
		
		for (SkirmishResult result : skirmishResults) {
			
			if (result.getOutcome() == Outcome.LOST){
				return false;
			}
		}
		
		return true;
		
	}
	
	public List<Unit> getRemainingGoodGuys() {
		
		List<Unit> remainingGoodGuys = new ArrayList<Unit>();
		
		if (goodGuysPicked != null && goodGuysPicked.size() > 0) {
			for (Unit unit : goodGuysPicked) {
				
				if(!unit.isEliminated()) {
					remainingGoodGuys.add(unit);
				}
			}
		}
		
		return remainingGoodGuys;
	}
	
	public List<Unit> getRemainingBadGuys() {
		
		List<Unit> remainingBadGuys = new ArrayList<Unit>();
		
		if (badGuysPicked != null && badGuysPicked.size() > 0) {
			for (Unit unit : badGuysPicked) {
				
				if(!unit.isEliminated()) {
					remainingBadGuys.add(unit);
				}
			}
		}
		
		return remainingBadGuys;
	}

	private Unit determineWinner(Unit unitA, Unit unitB) {
		
		return (unitA.score(unitB) > unitB.score(unitA)) ? unitA : unitB;
	}
	
	private double determineTroopsRemaining(Unit winningTeam, Unit losingTeam) {
		
		double realScore = BattleScores.determineRealScore(winningTeam, losingTeam);
		
		if (strongTeamWon(winningTeam, losingTeam)) {
			
			return Math.ceil(winningTeam.getTroopCount() - (losingTeam.getTroopCount() * realScore));
		} else {
			
			return Math.ceil(winningTeam.getTroopCount() - (losingTeam.getTroopCount() / realScore));
		}
	}
	
	private boolean strongTeamWon(Unit winningTeam, Unit losingTeam) {
		return (winningTeam == BattleScores.determineStrongTeam(winningTeam, losingTeam));
	}
}
